@@ -38,7 +38,7 @@ ggoutlier_geoKNN <- function(geo_coord,
                              cpu = 1,
                              verbose = TRUE
 ){
-  required_pkgs <- c("geosphere", # for calculating geographical distances
+  required_pkgs <- c("sf", # for calculating geographical distances
                      "stats4", # package to perform maximum likelihood estimation
                      "FastKNN", # KNN algorithm using a given distance matrix (other packages do not take arbitrary distance matrices)
                      "foreach", "doParallel",
@@ -98,10 +98,20 @@ ggoutlier_geoKNN <- function(geo_coord,
 
   #--------------------------------
   # Data pre-treatment
+
+  ## geographical data: data.frame to sf format
+  geo_data_df <- data.frame(ID = rownames(geo_coord),
+                            x = geo_coord$x,
+                            y = geo_coord$y)
+  geo_data_sf <- sf::st_as_sf(geo_data_df,
+                              coords = c("x", "y"), crs = 4326)
+
+
   ## calculate geographical distance
-  geo.dM <- geosphere::distm(x = geo_coord)/s
+  geo.dM <- sf::st_distance(geo_data_sf)/s
+  geo.dM <- matrix(as.vector(geo.dM), ncol = ncol(geo.dM), nrow = nrow(geo.dM)) # convert geo.dM from `units` to matrix
   ## handle samples with identical geographical coordinates
-  if(any(geo.dM[lower.tri(geo.dM)] == 0)){
+  if(any(as.vector(geo.dM[lower.tri(geo.dM)]) == 0)){
     if(verbose) cat("Find samples with identical geographical coordinates.\n")
     if(any(min_nn_dist == 0 , is.null(min_nn_dist))){
       if(verbose) cat("Add one unit of distance to individual pairs\n")
@@ -143,7 +153,12 @@ ggoutlier_geoKNN <- function(geo_coord,
       dev.off()
     }
 
-    if(verbose) cat(paste("\n The optimal k is ",opt.k,". Its figure is saved at ", k.sel.plot," \n", sep = ""))
+    if(make_fig){
+      if(verbose) cat(paste0("\n The optimal k is ",opt.k,". Its figure is saved at ", k.sel.plot," \n"))
+    }else{
+      if(verbose) cat(paste0("\n The optimal k is ",opt.k," \n"))
+    }
+
 
   }
   if(is.null(k)){stop("k is NULL!")}
@@ -249,6 +264,8 @@ ggoutlier_geoKNN <- function(geo_coord,
     # if `maxIter` is NULL -> let it equal to 50% of sample size
     if(is.null(maxIter)){maxIter <- round(nrow(gen_coord) * 0.5)}
     while (i <= maxIter) {
+      cat("iteration",i, "\r")
+      flush.console()
       if(i > 1){
         tmp.gen_coord <- tmp.gen_coord[to_keep,]
         tmp.geo.dM <- tmp.geo.dM[to_keep, to_keep]
