@@ -21,20 +21,21 @@
 #' @param  plot_ylim vector. Values controlling latitude boundaries of a map.
 #' @param  only_edges_in_xylim logic. only the edges with starting points within the given `select_xlim` and `select_ylim` will display on a geographical map. If `FALSE`, the edges out of the given boundaries will be removed from your plot. The default is `TRUE`.
 #' @param  pie_r_scale numeric. A scale controlling the radius of pie charts
-#' @param  red_alpha numeric. A value controlling the transparency of red lines. the default is 0.8
-#' @param  map_resolution string. The resolution of the geographical map. See details in the manual of `rworldmap::getMap()`
+#' @param  map_resolution a character string. The resolution of the geographical map. See details of the `scale` argument in the manual of `rnaturalearth::ne_countries()`. The default is `map_resolution = "medium"`
 #' @param  show_knn_pie logic. If `TRUE`, the ancestry coefficients of K nearest neighbors of significant samples will display on the map. The default is `FALSE`.
-#' @param  show_col_bar logic. If `TRUE`, a color key will be added to the output graph.
 #' @param  which_sample a string vector of sample ID(s). If users want to only show specific sample(s)
 #' @param  add_benchmark_graph logic. If `TRUE`, a benchmark graph with only pie charts of ancestry coefficients for comparison with the outlier graph.
-#' @param  vertical_plots logic. If `TRUE`, a benchmark graph and outlier graph will be combined in a vertical direction.
 #' @param  adjust_p_value_projection logic. If `TRUE`, the function will perform KNN prediction by forcing K=1 and compute new p-values for visualization.
-#' @returns a geographical map with colored lines showing sample pairs with unusual geo-genetic associations.
+#' @param  linewidth_range numeric. A vector of two values. It is used to control the minimal and maximal width of KNN network on the geographical map.
+#' @param  plot_labels character. A string of labels for plots. The default is `plot_labels = "auto"`. This parameter is for `cowplot::plot_grid`.
+#'
+#' @returns ggplot object. The plot is geographical map(s) with colored lines showing sample pairs with unusual geo-genetic associations.
 #' @examples
 #' library(GGoutlieR)
 #' data("ipk_anc_coef") # get ancestry coefficients
 #' data("ipk_geo_coord") # get geographical coordinates
 #' data(ggoutlier_example) # get an example output of ggoutlier
+#' \dontrun{
 #' plot_ggoutlier(ggoutlier_res = ggoutlier_example,
 #'                gen_coord = ipk_anc_coef,
 #'                geo_coord = ipk_geo_coord,
@@ -44,8 +45,9 @@
 #'                select_ylim = c(10,62),
 #'                plot_xlim = c(-20,140),
 #'                plot_ylim = c(10,62),
-#'                pie_r_scale = 1.2,
-#'                map_resolution = "coarse")
+#'                pie_r_scale = 1.8,
+#'                map_resolution = "medium")
+#'                }
 #'
 #' @export
 
@@ -61,7 +63,7 @@ plot_ggoutlier <- function(ggoutlier_res,
                            map_color = "black",
                            p_thres = NULL,
                            color_res = 10,
-                           dot_cex = 0.4,
+                           dot_cex = NULL,
                            map_type = c("geographic_knn", "genetic_knn", "both"),
                            select_xlim = c(-180,180),
                            select_ylim = c(-90,90),
@@ -69,20 +71,26 @@ plot_ggoutlier <- function(ggoutlier_res,
                            plot_ylim = NULL,
                            only_edges_in_xylim = TRUE,
                            pie_r_scale = 1,
-                           red_alpha = 0.8,
-                           map_resolution = "low",
+                           map_resolution = "medium",
                            show_knn_pie = FALSE,
-                           show_col_bar = TRUE,
                            which_sample = NULL,
                            add_benchmark_graph = TRUE,
-                           vertical_plots = TRUE,
-                           adjust_p_value_projection = FALSE
+                           adjust_p_value_projection = FALSE,
+                           linewidth_range = c(0.5,3),
+                           plot_labels = "auto"
+
 
 ){
-  required_pkgs <- c("rworldmap", "scales", "plotrix",
-                     "mapplots", "RColorBrewer",
-                     "rworldxtra","dichromat", "sp")
-  invisible(lapply(required_pkgs, FUN=function(x){suppressPackageStartupMessages(library(x, verbose = FALSE, character.only = TRUE))}))
+  required_pkgs <- c(#"rworldmap",
+                     #"mapplots",
+                     #"rworldxtra","dichromat", "sp",
+                     "RColorBrewer","scales",
+                     "rnaturalearth",
+                     "rnaturalearthdata", "sf",
+                     "ggplot2", "cowplot",
+                     "ggforce", "rlang", #"ggfun",
+                     "stats", "tidyr", "dplyr", "utils")
+  invisible(lapply(required_pkgs, FUN=function(x){suppressMessages(library(x, verbose = FALSE, character.only = TRUE))}))
 
   # use on.exit to prevent changes in users' pars
   oldpar <- par(no.readonly = TRUE) # save original par of users
@@ -103,6 +111,25 @@ plot_ggoutlier <- function(ggoutlier_res,
   }
 
   # check input
+  if(is.null(anc_coef) & is.null(gen_coord)){
+    stop("please provide at least either `anc_coef` or `gen_coord`")
+  }else{
+    if(is.null(gen_coord)){gen_coord <- anc_coef}
+    if(is.null(anc_coef) & !is.null(gen_coord)){
+      if(all(abs(apply(gen_coord,1 , sum) - 1) < 10^-5)){ # double check if gen_coord is as expectation of ancestry coefficients
+        message("assign `gen_coord` as ancestry coefficients (`anc_coef`)")
+        anc_coef <- gen_coord
+      }else{
+        message("\n ancestry coefficients are missing.\n")
+      }
+    } # if is.null(anc_coef) end
+  }
+  if(!is.null(anc_coef)){
+    if(!all(abs(apply(anc_coef,1 , sum) - 1) < 10^-5)){
+      stop("sum of ancestry coefficients of each individual should be 1. please check your data")
+    }
+  }
+
   map_type = match.arg(map_type)
   if(!is.null(anc_coef) & !is.null(p_thres)){
     if(map_type == "both" | map_type == "geographic_knn"){
@@ -122,16 +149,19 @@ plot_ggoutlier <- function(ggoutlier_res,
         stop("the sample size in `GenSP_knn_res` and `geo_coord` does not match!")
       }
     }
+    if(is.null(dot_cex)){
+      dot_cex <- min(linewidth_range)
+    }
 
     # assign colors if pie_color is NULL
     if(!is.null(anc_coef) & is.null(pie_color)){
       # check if sample sizes match
       if(nrow(anc_coef) != nrow(geo_coord)){stop("the sample size in `anc_coef` and `geo_coord` does not match!")}
       if(ncol(anc_coef) <= 12){
-        pie_color = brewer.pal(ncol(anc_coef), "Set3")
+        pie_color = RColorBrewer::brewer.pal(ncol(anc_coef), "Set3")
       }else{
         # color generation: https://stackoverflow.com/questions/15282580/how-to-generate-a-number-of-most-distinctive-colors-in-r
-        qual_col_pals = brewer.pal.info[brewer.pal.info$category == 'qual',]
+        qual_col_pals = RColorBrewer::brewer.pal.info[brewer.pal.info$category == 'qual',]
         col_vector = unlist(mapply(brewer.pal, qual_col_pals$maxcolors, rownames(qual_col_pals)))
         # shuffle colors
         pie_color = c(matrix(c(matrix(col_vector[1:72], ncol = 8, byrow = T))[c(matrix(c(1:36, rev(37:72)), nrow = 3))], ncol = 4,byrow = T))
@@ -144,18 +174,22 @@ plot_ggoutlier <- function(ggoutlier_res,
     warning("`p_thres` is `NULL`. ancestry coefficients will NOT be projected to your geographical map. the pie charts of outliers could be added to the map if you give `p_thres`.")
   }
 
-  if(is.null(anc_coef) & is.null(gen_coord)){
-    stop("please provide at least either `anc_coef` or `gen_coord`")
-  }else{
-    if(is.null(gen_coord)){gen_coord <- anc_coef}
-    if(is.null(anc_coef)){anc_coef <- gen_coord}
-  }
 
-  geomap <- getMap(resolution = map_resolution)
+
+  ## geographical data: data.frame to sf format
+  geo_data_df <- data.frame(ID = rownames(geo_coord),
+                            x = geo_coord$x,
+                            y = geo_coord$y)
+
+  geo_data_sf <- sf::st_as_sf(geo_data_df,
+                              coords = c("x", "y"), crs = 4326)
+  # get world geographical map
+  geomap <- rnaturalearth::ne_countries(scale = map_resolution, returnclass = "sf")
+  #geomap <- getMap(resolution = map_resolution)
 
   # get an adjancency matrix 'GGNet_adjm'
   if(all(dim(gen_coord) == dim(anc_coef))){
-    if(all(round(anc_coef, digits = 5) == round(anc_coef, digits = 5))){
+    if(all(round(gen_coord, digits = 5) == round(anc_coef, digits = 5))){
       gen_coord_eq_AncCoeff = TRUE
     }else{gen_coord_eq_AncCoeff = FALSE}
   }
@@ -215,7 +249,7 @@ plot_ggoutlier <- function(ggoutlier_res,
   ## set plot_xlim and plot_ylim if NULL
   if(is.null(plot_xlim)){plot_xlim <- select_xlim}
   if(is.null(plot_ylim)){plot_ylim <- select_ylim}
-  par(mar = c(1,1,1,1))
+  #par(mar = c(1,1,1,1))
   #******************************
   # edges of genetic space (GenSP) KNN outlier likelihood
   if(map_type %in% c("both", "genetic_knn")){
@@ -259,199 +293,286 @@ plot_ggoutlier <- function(ggoutlier_res,
       gensp.df <- gensp.df[which(to_keep),]
     }
   }
+  if(!is.null(anc_coef)){
+    ## prepare a data frame for making pie charts on ggplot
+    anc_coef_for_pie <- data.frame(geo_coord,
+                                   pie_r_scale = pie_r_scale,
+                                   anc_coef
+    )
+    cols <- colnames(anc_coef_for_pie)[4:ncol(anc_coef_for_pie)]
+    cols2 <- enquo(cols)
+    anc_ggplot_df <- gather(anc_coef_for_pie, "type", "value", !!cols2)
+    anc_ggplot_df$type <- factor(anc_ggplot_df$type, levels = cols) # set legend order based on order of "cols"
+  }
 
   if(add_benchmark_graph & !is.null(anc_coef)){
-    if(vertical_plots){
-      par(mfrow = c(2,1), mar = c(1,1,1,1))
-    }else{
-      par(mfrow = c(1,2), mar = c(1,1,1,1))
-    }
-    sp::plot(geomap, xlim = plot_xlim, ylim = plot_ylim, border = map_color)
-    pie.r = abs(diff(par("usr")[1:2]) )*0.005
-    for(i in 1:nrow(anc_coef)){
-      add.pie(z = round(anc_coef[i,]*10^5),
-              x = geo_coord[i,1],
-              y = geo_coord[i,2],
-              col = pie_color,
-              labels = NA,
-              radius = pie.r*pie_r_scale)
-    }
-  }
+    #if(vertical_plots){
+      #par(mfrow = c(2,1), mar = c(1,1,1,1))
+    #}else{
+      #par(mfrow = c(1,2), mar = c(1,1,1,1))
+    #}
 
-  ## make a blank map
-  sp::plot(geomap, xlim = plot_xlim, ylim = plot_ylim, col = NA, border = "white")
+
+    ## make benchmark plot
+    geomap_plot_benchmark <-
+      ggplot2::ggplot(data = geomap) +
+      ggplot2::geom_sf(color = map_color, show.legend = NA) +
+      coord_sf(xlim = plot_xlim, ylim = plot_ylim, expand = TRUE) +
+
+      scale_fill_manual(values=pie_color) +
+      geom_arc_bar(aes(x0 = .data$x, y0 = .data$y,r0=0,
+                       fill = .data$type, r=pie_r_scale,
+                       amount= .data$value),
+                   data=anc_ggplot_df, stat='pie',
+                   col = "black",
+                   show.legend = FALSE) +
+      theme(axis.title.x = element_text(color = "white"),
+            axis.title.y = element_text(color = "white"),
+            # hide legend but keep blank space
+            legend.text = element_text(color = "white"),
+            legend.title = element_text(color = "white"),
+            legend.key = element_rect(fill = NA)
+            ) +
+      guides(color = guide_legend(override.aes =
+                                    list(fill = NA,
+                                         color = c("white"),
+                                         alpha = 0)))
+      #print(geomap_plot_benchmark)
+
+
+    #save.image("tmp_test_sf.Rdata")
+
+
+    #sp::plot(geomap, xlim = plot_xlim, ylim = plot_ylim, border = map_color)
+    #pie.r = abs(diff(par("usr")[1:2]) )*0.005
+    #for(i in 1:nrow(anc_coef)){
+    #  add.pie(z = round(anc_coef[i,]*10^5),
+    #          x = geo_coord[i,1],
+    #          y = geo_coord[i,2],
+    #          col = pie_color,
+    #          labels = NA,
+    #          radius = pie.r*pie_r_scale)
+    #}
+  } # make benchmark plot end
+
+  ##---------------------------------------------------------
+  ## prepare ggplot input for the GGoutlieR network plot(s)
+  ##---------------------------------------------------------
+
+  #sp::plot(geomap, xlim = plot_xlim, ylim = plot_ylim, col = NA, border = "white")
   if(map_type == "both" | map_type == "geographic_knn"){
     if(is.null(p_thres)){
-      segments(x0 = geo_coord[geosp.df$row,1],
-               y0 = geo_coord[geosp.df$row,2],
-               x1 = geo_coord[geosp.df$col,1],
-               y1 = geo_coord[geosp.df$col,2],
-               col = alpha(geosp.df$geosp.col, alpha = 0.9),
-               lwd = -log10(geosp.df$geosp.pvalue)
-      )
+
+      ## create a data.frame for drawing segments on ggplot
+      ## x0 and y0 correspond to the rows (the samples under testing)
+      ## x1 and y1 correspond to the columns (the K nearest neighbors of the samples under testing)
+      link_geosp_df <-
+        data.frame(
+          x0 = geo_coord[geosp.df$row,1],
+          y0 = geo_coord[geosp.df$row,2],
+          x1 = geo_coord[geosp.df$col,1],
+          y1 = geo_coord[geosp.df$col,2],
+          logp = round(-log10(geosp.df$geosp.pvalue), digits = 2)
+        )
+
     }else{
-      # sig.row corresponds to the rows (the samples under testing)
-      # sig.col corresponds to the columns (the K nearest neighbors of the samples under testing)
-      sig.row <- geosp.df$row[geosp.df$sig.tag]
-      sig.col <- geosp.df$col[geosp.df$sig.tag]
-      sig.color <- geosp.df$geosp.col[geosp.df$sig.tag]
-      sig.likelihood <- geosp.df$geosp.pvalue[geosp.df$sig.tag]
-      segments(x0 = geo_coord[sig.row,1],
-               y0 = geo_coord[sig.row,2],
-               x1 = geo_coord[sig.col,1],
-               y1 = geo_coord[sig.col,2],
-               col = alpha(sig.color, alpha = 0.9),
-               lwd = -log10(sig.likelihood)
-      )
+
+      ## create a data.frame for drawing segments on ggplot
+      ## x0 and y0 correspond to the rows (the samples under testing)
+      ## x1 and y1 correspond to the columns (the K nearest neighbors of the samples under testing)
+      link_geosp_df <-
+        data.frame(
+          x0 = geo_coord[geosp.df$row[geosp.df$sig.tag],1],
+          y0 = geo_coord[geosp.df$row[geosp.df$sig.tag],2],
+          x1 = geo_coord[geosp.df$col[geosp.df$sig.tag],1],
+          y1 = geo_coord[geosp.df$col[geosp.df$sig.tag],2],
+          logp = round(-log10(geosp.df$geosp.pvalue[geosp.df$sig.tag]), digits = 2)
+        )
     }
-  }
+
+  } # if map_type == "both" or "geographic_knn" end
   if(map_type == "both" | map_type == "genetic_knn"){
     # adjust transparency according to the type of figures
-    if(map_type == "both"){
-      tmp.alpha = red_alpha * 0.75
-    }else{
-      tmp.alpha = red_alpha
-    }
     if(is.null(p_thres)){
-      segments(x0 = geo_coord[gensp.df$row,1],
-               y0 = geo_coord[gensp.df$row,2],
-               x1 = geo_coord[gensp.df$col,1],
-               y1 = geo_coord[gensp.df$col,2],
-               col = alpha(gensp.df$gensp.col, alpha = tmp.alpha),
-               lwd = -log10(gensp.df$gensp.pvalue)
-      )
+      ## create a data.frame for drawing segments on ggplot
+      ## x0 and y0 correspond to the rows (the samples under testing)
+      ## x1 and y1 correspond to the columns (the K nearest neighbors of the samples under testing)
+      link_gensp_df <-
+        data.frame(
+          x0 = geo_coord[gensp.df$row,1],
+          y0 = geo_coord[gensp.df$row,2],
+          x1 = geo_coord[gensp.df$col,1],
+          y1 = geo_coord[gensp.df$col,2],
+          #col = alpha(gensp.df$gensp.col, alpha = tmp.alpha),
+          logp = round(-log10(gensp.df$gensp.pvalue), digits = 2)
+        )
+
     }else{
+      ## create a data.frame for drawing segments on ggplot
+      ## x0 and y0 correspond to the rows (the samples under testing)
+      ## x1 and y1 correspond to the columns (the K nearest neighbors of the samples under testing)
+      link_gensp_df <-
+        data.frame(
+          x0 = geo_coord[gensp.df$row[gensp.df$sig.tag],1],
+          y0 = geo_coord[gensp.df$row[gensp.df$sig.tag],2],
+          x1 = geo_coord[gensp.df$col[gensp.df$sig.tag],1],
+          y1 = geo_coord[gensp.df$col[gensp.df$sig.tag],2],
+          #col = alpha(gensp.df$gensp.col[gensp.df$sig.tag], alpha = tmp.alpha),
+          logp = round(-log10(gensp.df$gensp.pvalue[gensp.df$sig.tag]), digits = 2)
+        )
+
       # sig.row corresponds to the rows (the samples under testing)
       # sig.col corresponds to the columns (the K nearest neighbors of the samples under testing)
-      sig.row <- gensp.df$row[gensp.df$sig.tag]
-      sig.col <- gensp.df$col[gensp.df$sig.tag]
-      sig.color <- gensp.df$gensp.col[gensp.df$sig.tag]
-      sig.likelihood <- gensp.df$gensp.pvalue[gensp.df$sig.tag]
-      segments(x0 = geo_coord[sig.row,1],
-               y0 = geo_coord[sig.row,2],
-               x1 = geo_coord[sig.col,1],
-               y1 = geo_coord[sig.col,2],
-               col = alpha(sig.color, alpha = tmp.alpha),
-               lwd = -log10(sig.likelihood)
-      )
     }
   }
 
-  sp::plot(geomap, xlim = plot_xlim, ylim = plot_ylim, add=T, border = map_color)
-  if(is.null(anc_coef)){points(x = geo_coord[,1], y = geo_coord[,2], pch = 16, cex = dot_cex)}
-  if(!is.null(anc_coef)){
-    points(x = geo_coord[,1], y = geo_coord[,2], pch = 16, cex = dot_cex)
-    #}else{
-    if(!is.null(p_thres) & map_type %in% c("both","geographic_knn")){
+  ##---------------------------------------------
+  ## make ggplot
+  ## make a blank map
+  geomap_plot <-
+    ggplot2::ggplot(data = geomap) +
+    ggplot2::geom_sf(color = map_color) +
+    coord_sf(xlim = plot_xlim, ylim = plot_ylim, expand = TRUE) +
+    theme(axis.title.x = element_text(color = "white"),
+          axis.title.y = element_text(color = "white")
+    )
+  ## add links of geographic KNN to geographical map
+  if(map_type == "both" | map_type == "geographic_knn"){
+    geomap_plot_GeoSpKNN <-
+      geomap_plot +
+      geom_segment(aes(x=.data$x0,
+                       y=.data$y0,
+                       xend=.data$x1,
+                       yend=.data$y1,
+                       alpha = .data$logp,
+                       linewidth = .data$logp
+      ),
+      data=link_geosp_df,
+      lineend = "round",
+      show.legend = NA,
+      inherit.aes=FALSE, color = "blue") +
+      labs(linewidth =bquote("-"~log[10]~"(p)"),
+           alpha = bquote("-"~log[10]~"(p)"))+
+      scale_linewidth(range = linewidth_range) +
+      geom_point(mapping = aes(x = .data$x, y = .data$y), data = geo_coord, size = dot_cex)
+
+    ## add pies
+    if(!is.null(p_thres) & !is.null(anc_coef)){
       sig.row <- unique(geosp.df$row[geosp.df$sig.tag])
       if(length(sig.row) == 0){
-          warning("there is no outlier in the region selected according to `select_xlim` and `select_ylim`")
+        warning("there is no outlier in the region selected according to `select_xlim` and `select_ylim`")
       }else{
-        pie.r = abs(diff(par("usr")[1:2]) )*0.005
-        for(i in 1:length(sig.row)){
-          add.pie(z = round(anc_coef[sig.row[i],]*10^5),
-                  x = geo_coord[sig.row[i],1],
-                  y = geo_coord[sig.row[i],2],
-                  col = pie_color,
-                  labels = NA,
-                  radius = pie.r*pie_r_scale)
-        }
-        # add pie charts of KNNs if show_knn_pie is TRUE
-        if(show_knn_pie){
-          for(i in 1:length(sig.row)){
-            for(j in GeoSP_knn_res$knn_index[sig.row[i],]){
-              add.pie(z = round(anc_coef[j,]*10^5),
-                      x = geo_coord[j,1],
-                      y = geo_coord[j,2],
-                      col = pie_color,
-                      labels = NA,
-                      radius = pie.r*pie_r_scale)
-            }
-          }
-        }
-      }
-    }
-    if(!is.null(p_thres)  & map_type %in% c("both","genetic_knn")){
+
+        if(show_knn_pie){# add pie charts of KNNs if show_knn_pie is TRUE
+          sig_knn_index <- unique(unlist(lapply(sig.row, function(i){GeoSP_knn_res$knn_index[i,]})))
+          sig.row <- c(sig.row, sig_knn_index)
+        } # show_knn_pie end
+
+        sig_anc_ggplot_df <- gather(anc_coef_for_pie[sig.row,], "type", "value", !!cols2)
+        sig_anc_ggplot_df$type <- factor(sig_anc_ggplot_df$type, levels = cols) # set legend order based on order of "cols"
+        geomap_plot_GeoSpKNN <-
+          geomap_plot_GeoSpKNN +
+          scale_fill_manual(values=pie_color) +
+          geom_arc_bar(aes(x0 = .data$x, y0 = .data$y,r0=0,
+                           fill = .data$type, r=pie_r_scale,
+                           amount=.data$value),
+                       data=sig_anc_ggplot_df, stat='pie',
+                       col = "black", show.legend = FALSE)
+      } # if no significant end
+    } # add pie Geographic KNN end
+  } # make geomap_plot_GeoSpKNN end
+  if(map_type == "both" | map_type == "genetic_knn"){
+    geomap_plot_GeneticKNN <-
+      geomap_plot +
+      geom_segment(aes(x=.data$x0,
+                       y=.data$y0,
+                       xend=.data$x1,
+                       yend=.data$y1,
+                       alpha = .data$logp,
+                       linewidth = .data$logp
+      ),
+      data=link_gensp_df,
+      lineend = "round",
+      show.legend = NA,
+      inherit.aes=FALSE, color = "red") +
+      labs(linewidth =bquote("-"~log[10]~"(p)"),
+           alpha = bquote("-"~log[10]~"(p)"))+
+      scale_linewidth(range = linewidth_range) +
+      geom_point(mapping = aes(x = .data$x, y = .data$y), data = geo_coord, size = dot_cex)
+
+    ## add pies
+    if(!is.null(p_thres) & !is.null(anc_coef)){
       sig.row <- unique(gensp.df$row[gensp.df$sig.tag])
       if(length(sig.row) == 0){
         warning("there is no outlier in the region selected according to `select_xlim` and `select_ylim`")
       }else{
-        pie.r = abs(diff(par("usr")[1:2]) )*0.005
-        for(i in 1:length(sig.row)){
-          add.pie(z = round(anc_coef[sig.row[i],]*10^5),
-                  x = geo_coord[sig.row[i],1],
-                  y = geo_coord[sig.row[i],2],
-                  col = pie_color,
-                  labels = NA,
-                  radius = pie.r*pie_r_scale)
-        }
-        # add pie charts of KNNs if show_knn_pie is TRUE
-        if(show_knn_pie){
-          for(i in 1:length(sig.row)){
-            for(j in GenSP_knn_res$knn_index[sig.row[i],]){
-              add.pie(z = round(anc_coef[j,]*10^5),
-                      x = geo_coord[j,1],
-                      y = geo_coord[j,2],
-                      col = pie_color,
-                      labels = NA,
-                      radius = pie.r*pie_r_scale)
-            }
-          }
-        }
-      }
+        if(show_knn_pie){# add pie charts of KNNs if show_knn_pie is TRUE
+          sig_knn_index <- unique(unlist(lapply(sig.row, function(i){GenSP_knn_res$knn_index[i,]})))
+          sig.row <- c(sig.row, sig_knn_index)
+        } # show_knn_pie end
+
+        sig_anc_ggplot_df <- gather(anc_coef_for_pie[sig.row,], "type", "value", !!cols2)
+        sig_anc_ggplot_df$type <- factor(sig_anc_ggplot_df$type, levels = cols) # set legend order based on order of "cols"
+        geomap_plot_GeneticKNN <-
+          geomap_plot_GeneticKNN +
+          scale_fill_manual(values=pie_color) +
+          geom_arc_bar(aes(x0 = .data$x, y0 = .data$y,r0=0,
+                           fill = .data$type, r=pie_r_scale,
+                           amount=.data$value),
+                       data=sig_anc_ggplot_df, stat='pie',
+                       col = "black", show.legend = FALSE)
+      } # if no significant end
+    } # add pie Genetic KNN end
+  } # make geomap_plot_GeneticKNN end
+
+  ##-----------------------------------
+  ## output plot
+  ## -> combine plots using `cowplot`
+  if(map_type == "both"){
+    if(add_benchmark_graph){
+      plot_out <-
+        cowplot::plot_grid(
+          geomap_plot_benchmark,
+          geomap_plot_GeneticKNN,
+          geomap_plot_GeoSpKNN,
+          labels = plot_labels, ncol = 1
+       )
+    }else{
+      plot_out <-
+        cowplot::plot_grid(
+          geomap_plot_GeneticKNN,
+          geomap_plot_GeoSpKNN,
+          labels = plot_labels, ncol = 1
+        )
+    }
+    return(plot_out)
+  } # if both end
+  if(map_type == "genetic_knn"){
+    if(add_benchmark_graph){
+      plot_out <-
+        cowplot::plot_grid(
+          geomap_plot_benchmark,
+          geomap_plot_GeneticKNN,
+          labels = plot_labels, ncol = 1
+        )
+      return(plot_out)
+    }else{
+      return(geomap_plot_GeneticKNN)
+    }
+  } # if genetic_knn end
+  if(map_type == "geographic_knn"){
+    if(add_benchmark_graph){
+      plot_out <-
+        cowplot::plot_grid(
+          geomap_plot_benchmark,
+          geomap_plot_GeoSpKNN,
+          labels = plot_labels, ncol = 1
+        )
+      return(plot_out)
+    }else{
+      return(geomap_plot_GeoSpKNN)
     }
   }
-  if(map_type == "genetic_knn" & show_col_bar){
-    color.legend( xl =plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.15,
-                  xr = plot_xlim[2],
-                  yb = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.07,
-                  yt = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.05 , # the coordinates
-                  legend = c(0, round(max(-log10(gensp.pvalue)))) ,
-                  gradient="x",
-                  rect.col=edge.col$GenSP_colkey, align="rb")
-    text(x = plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.075,
-         y = plot_ylim[2],
-         labels = expression(-log[10](p)), font = 2, cex = 1.1
-    )
-  }
-
-  if(map_type == "geographic_knn" & show_col_bar){
-    color.legend( xl =plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.15,
-                  xr = plot_xlim[2],
-                  yb = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.07,
-                  yt = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.05 , # the coordinates
-                  legend = c(0, round(max(-log10(geosp.pvalue)))) ,
-                  gradient="x",
-                  rect.col=edge.col$GeoSP_colkey, align="rb")
-    text(x = plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.075,
-         y = plot_ylim[2],
-         labels = expression(-log[10](p)), font = 2, cex = 1.1
-    )
-  }
-  if(map_type == "both" & show_col_bar){
-    par(xpd = NA)
-    color.legend( xl =plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.15,
-                  xr = plot_xlim[2],
-                  yb = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.06,
-                  yt = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.03 , # the coordinates
-                  legend = c(0, round(max(-log10(geosp.pvalue)))) ,
-                  gradient="x",
-                  rect.col=edge.col$GeoSP_colkey, align="rb")
-    color.legend( xl =plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.15,
-                  xr = plot_xlim[2],
-                  yb = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.15,
-                  yt = plot_ylim[2] - abs(plot_ylim[2] - plot_ylim[1])*0.12, # the coordinates
-                  legend = c(0, round(max(-log10(gensp.pvalue)))) ,
-                  gradient="x",
-                  rect.col=edge.col$GenSP_colkey, align="rb")
-
-    text(x = plot_xlim[2] - abs(plot_xlim[2] - plot_xlim[1])*0.075,
-         y = plot_ylim[2] + abs(plot_ylim[2] - plot_ylim[1])*0.02,
-         labels = expression(-log[10](p)), font = 2, cex = 1.1
-    )
-    par(xpd = FALSE)
-  }
-  par(mfrow = c(1,1), mar = c(4,4,4,4)) # recover plot setting
 } # plot_ggoutlier end
 
 
